@@ -1,6 +1,7 @@
 # hidden.R
+# Utility functions for MSBVAR package
+#
 # Patrick T. Brandt
-# brandt@unt.edu
 
 # This file contains utility functions for VAR models.  These
 # functions are not really meant to be called by users.  In general,
@@ -12,13 +13,15 @@
 
 # Compute VAR forecasts using the coefficients that have been parsed
 # from the VAR object.
-"coef.forecast.var" <-
-function(y, intercept, ar.coefs, exog.coefs, m, p,
-                       capT, nsteps, A0=t(chol(var.obj$mean.S)),
-                       shocks=matrix(0,nrow=nsteps,ncol=m),
-                       exog.fut=matrix(0,nrow=nsteps, ncol=nrow(exog.coefs)))
+
+"coef.forecast.VAR" <- function(object, intercept, ar.coefs,
+                                exog.coefs, m, p, capT, nsteps,
+                                A0=t(chol(var.obj$mean.S)),
+                                shocks=matrix(0,nrow=nsteps,ncol=m),
+                                exog.fut=matrix(0,nrow=nsteps,ncol=nrow(exog.coefs)),
+                                ...)
 {
-  yhat<-rbind(y,matrix(0,ncol=m,nrow=nsteps))
+  yhat<-rbind(object,matrix(0,ncol=m,nrow=nsteps))
 
    # Compute the deterministic part of the forecasts (less the intercept!)
    if(is.na(sum(exog.coefs))==F)
@@ -55,7 +58,7 @@ function(A0,bvec,nsteps)
     ar.coefs<-aperm(ar.coefs,c(2,1,3))   # reorder array so columns
                                          # are for eqn
 
-    impulses <- irf.var(list(ar.coefs=ar.coefs),nsteps,A0=A0)$mhat
+    impulses <- irf.VAR(list(ar.coefs=ar.coefs),nsteps,A0=A0)$mhat
     impulses <- aperm(impulses, c(3,1,2)) # flips around the responses
                                           # to stack them as series
                                           # for each response, so the
@@ -149,7 +152,7 @@ function(capT,m,ncoef,num.exog,nu,H0,S0,Y,X,hstar1,Sh,u, Bh,Sh1)
 # Marginal Log-Posterior for A0 -- returns a single value.  Used for
 # optimization of the peak of the posterior pdf for B-SVAR model.
 
-# Inputs are part of the moments generated by STRUCTURAL.SZBVAR.
+# Inputs are part of the moments generated by szbsvar().
 # b = free parameter vector in A0
 # Ui = U matrices that map from a to b
 # df = degrees of freedom : T - p + ndum
@@ -170,9 +173,9 @@ function(capT,m,ncoef,num.exog,nu,H0,S0,Y,X,hstar1,Sh,u, Bh,Sh1)
           llf <- llf + 0.5*(t(bj)%*%H0inv.posterior[[i]]%*%bj)
       }
 
-      tmp <- lu(Matrix(t(A0), sparse=FALSE))
-      L <- matrix(expand(tmp)$L@x, m, m)
-      U <- matrix(expand(tmp)$U@x, m, m)
+#      tmp <- lu(Matrix(t(A0), sparse=FALSE))
+#      U <- matrix(expand(tmp)$U@x, m, m)
+      U <- qr.R(qr(t(A0)))
       llf <- llf - df*sum(log(abs(diag(U))))
       return(llf)
 }
@@ -185,7 +188,7 @@ function(capT,m,ncoef,num.exog,nu,H0,S0,Y,X,hstar1,Sh,u, Bh,Sh1)
 # footnote 11 of W&Z 2003.
 
 "b2a" <- function(b, Ui)
-{ m <- nrow(Ui[[1]])
+{ m <- nrow(as.matrix(Ui[[1]]))
   n0 <- sapply(1:m, function(i) {ncol(as.matrix(Ui[[i]]))})
   n0cum <- c(0,cumsum(n0))
   A0 <- matrix(0,m,m)
@@ -213,8 +216,8 @@ function(capT,m,ncoef,num.exog,nu,H0,S0,Y,X,hstar1,Sh,u, Bh,Sh1)
   }
 
 
-# Set up function for the Gibbs sampler for A0 for structural.szbvar
-# and gibbs.A0.  Input is a fitted structural.szbvar() object.
+# Set up function for the Gibbs sampler for A0 for szbsvar()
+# and gibbs.A0().  Input is a fitted szbsvar() object.
 
 "gibbs.setup.bsvar" <- function(szbvar.obj)
   {  # Compute some objects we need for inputs for the Gibbs sampler
@@ -227,7 +230,7 @@ function(capT,m,ncoef,num.exog,nu,H0,S0,Y,X,hstar1,Sh,u, Bh,Sh1)
                    {chol(szbvar.obj$H0inv.posterior[[i]])/szbvar.obj$df},
                    simplify=F)
 
-    # UT = m x qi matriX in equation 14 of WZ JEDC --- eqn 14.
+    # UT = m x qi matrix in equation 14 of WZ JEDC --- eqn 14.
     UT <- sapply(1:m, function(i)
                  {szbvar.obj$Ui[[i]]%*%solve(Tinv[[i]])},
                  simplify=F)
@@ -243,9 +246,98 @@ function(capT,m,ncoef,num.exog,nu,H0,S0,Y,X,hstar1,Sh,u, Bh,Sh1)
   }
 
 
+# Functions to "flatten" or conserve A0 storage and rebuild the
+# A0 matrices from the "flat" objects
+
+"A0.flatten" <- function(A0){
+    struct <- which(A0 != 0)
+    A0.flat <- as.vector(A0)[struct]
+    return(A0.flat)
+}
+
+"A0.get" <- function(A0,index){
+    if(index==1) {st <- 1} else {st <- 1 + (index-1)*length(A0$struct)}
+    A0.out <- vector(mode="numeric",length=(A0$m*A0$m))
+    A0.out[A0$struct] <- A0$A0[st:(st+length(A0$struct)-1)]
+    return(matrix(A0.out,nrow=A0$m))
+}
+
+# Functions to "flatten" or conserve W storage and rebuild the W
+# matrices from the "flat" objects
+
+"W.flatten" <- function(W){
+    m <- length(W)
+    W.flat <- vector(mode="numeric", length=((m-1)*(m-1)*m))
+    W.index <- vector(mode="numeric", length=m)
+    W.index[1] <- dim(W[[1]])[1]*dim(W[[1]])[2]
+    W.flat[1:W.index[1]] <- as.vector(W[[1]])
+    for(i in 2:m){
+      W.index[i] <- dim(W[[i]])[1]*dim(W[[i]])[2] + W.index[i-1]
+      W.flat[(W.index[i-1]+1):W.index[i]] <- as.vector(W[[i]])
+    }
+    return(list(W=W.flat[1:W.index[m]], W.index=W.index, m=m))
+}
+
+"Wlist.flatten" <- function(W){
+    N2 <- length(W)
+    m <- length(W[[1]])
+    W.flat <- vector(mode="numeric", length=((m-1)*(m-1)*m*N2))
+    W.index <- vector(mode="numeric", length=(m*N2))
+    agg.index <- 0
+    for(i in 1:N2){
+        for(j in 1:m){
+            dim.w <- dim(W[[i]][[j]])[1]
+            cur.index <- (((i-1)*m)+j)
+            if(i==1 && j==1){agg.index <- 0}else{agg.index <- W.index[(cur.index-1)]}
+            W.index[cur.index] <- ((dim.w*dim.w)+agg.index)
+            W.flat[(agg.index+1):W.index[cur.index]] <- as.vector(W[[i]][[j]])
+        }
+    }
+    return(list(W=W.flat[1:max(W.index)], W.index=W.index, m=m))
+}
+
+"W.list" <- function(W){
+    m <- W$m
+    N2 <- length(W$W.index)/m
+    W.flat <- W$W
+    W.index <- W$W.index
+    rm(W)
+    W.list <- vector(mode="list",length=N2)
+    W.tmp <- vector(mode="list",length=m)
+    for(i in 1:N2){
+        for(j in 1:m){
+            if(i==1 && j==1){last.index <- 0}else{last.index <- W.index[(((i-1)*m)+(j-1))]}
+            cur.index <- W.index[(((i-1)*m)+j)]
+            dim.w <- sqrt(cur.index-last.index)
+            W.tmp[[j]] <- matrix(W.flat[(last.index+1):cur.index],nrow=dim.w)
+        }
+        W.list[[i]] <- W.tmp
+    }
+    return(W.list)
+}
+
+"W.get" <- function(W,index)
+{
+    start.index <- (index-1) * W$m + 1
+    end.index <- start.index + W$m - 1
+    Wout <- vector(mode="list",length=W$m)
+    if(index==1){tmp.index <- 0}else{tmp.index <- W$W.index[(start.index-1)]}
+    for(i in start.index:end.index){
+        dim.w <- sqrt(W$W.index[i]-tmp.index)
+        w <- matrix(W$W[(tmp.index+1):W$W.index[i]],nrow=dim.w)
+        Wout[[(i-start.index+1)]] <- w
+        tmp.index <- W$W.index[i]
+    }
+    return(Wout)
+}
+
+
+
 # ONE STEP Function to draw 1 A0 from the posterior of a structural
 # BVAR model.  Can be used for the over and just identified cases.
 # Inputs all come from setup function above or the model object.
+
+# New version with memory conservation for A0's and W's
 "drawA0" <- function(A0gbs, UT, df, n0, Wout)
   { # Set up constants and place holder matrices
     m <- dim(A0gbs)[1]
@@ -256,19 +348,30 @@ function(capT,m,ncoef,num.exog,nu,H0,S0,Y,X,hstar1,Sh,u, Bh,Sh1)
 
         X <- A0gbs
         X[,i] <- 0
-        tmp <- lu(Matrix(t(X), sparse=FALSE))
+        tmp <- qr(t(X), tol=1e-12)  # replaces the LU call from above
 
-        L <- matrix(expand(tmp)$L@x, m, m)
-        U <- matrix(expand(tmp)$U@x, m, m)
-        d.U <- abs(diag(U))
+
+        # Now, get back the upper part of the decomposition for backsolving the
+        # system. We do not need the lower part
+
+        U <- qr.R(tmp)  # gets the upper triangular part of the QR decomp
+
+        d.U <- abs(diag(U))            # find the singular values
         sing.values <- d.U/max(d.U)
-        jIx0 <- min(which(sing.values<2^-30))
+        sing.index <- min(which(sing.values<2^-16)) # find the index of the
+                                                    # singular values for
+                                                    # the pivoted solution
+
+        jIx0 <- tmp$pivot[sing.index]  # need the location of the pivots / equations
+
         w[jIx0] <- 1
         if(jIx0>1 && jIx0<(m+1))
           {
             jA <- U[1:(jIx0-1), 1:(jIx0-1)]
-            jb <- U[1:(jIx0-1), jIx0]
-            jy <- qr.solve(-jA, jb)
+            jb <- U[1:(jIx0-1), sing.index]  # this is the singular part,
+                                             # so it is at the end of the matrix
+
+            jy <- qr.solve(-jA, jb, tol=1e-12)
             w[1:(jIx0-1)] <- jy
           }
 
@@ -282,13 +385,24 @@ function(capT,m,ncoef,num.exog,nu,H0,S0,Y,X,hstar1,Sh,u, Bh,Sh1)
      # previous ones in this error trap is probably OK.  If nothing
      # else, it increases the numerical stability of the algorithm
 
+        # Check that we were able to get a new orthnormal basis.  If
+        # not, reuse the last one and find its QR
+
         if(is.na(sum(w1))==TRUE)
-        {    W <- Wout[[i]]
+        {
+            W <- Wout[[1]]
+            print("is.na(sum(w1))!")
           } else {
               W <- qr.Q(qr(w1, tol=1e-12), complete=T)
           }
 
-        if(is.numeric(W)==FALSE) W <- Wout[[i]]
+        # Now trap that we got a valid QR.
+
+        if(is.numeric(W)==FALSE) {
+            print("Invalid qr.Q!")
+            W <- Wout[[i]]
+            print(W)
+        }
 
         gkb <- matrix(0, n0[i], 1)
         jstd <- sqrt(1/df)
@@ -301,8 +415,83 @@ function(capT,m,ncoef,num.exog,nu,H0,S0,Y,X,hstar1,Sh,u, Bh,Sh1)
                            -1*sqrt(crossprod(jr)))
         A0gbs[,i] <- UT[[i]]%*%(W%*%gkb)
         Wout[[i]] <- W
-      }
+    }
     return(list(A0gbs=A0gbs, W=Wout))
+}
 
-  }
+# old version, without memory conservation
+## "drawA0" <- function(A0gbs, UT, df, n0, Wout)
+##   { # Set up constants and place holder matrices
+##     m <- dim(A0gbs)[1]
+
+##     for(i in 1:m)
+##       {
+##         w <- matrix(0, m, 1)
+
+##         X <- A0gbs
+##         X[,i] <- 0
+##         tmp <- qr(t(X), tol=1e-12)
+
+##         # Now, get back the upper part of the decomposition for backsolving the
+##         # system. We do not need the lower part
+
+##         U <- qr.R(tmp)  # gets the upper triangular part of the QR decomp
+
+##         d.U <- abs(diag(U))            # find the singular values
+##         sing.values <- d.U/max(d.U)
+##         sing.index <- min(which(sing.values<2^-16)) # find the index of the
+##                                                     # singular values for
+##                                                     # the pivoted solution
+
+##         jIx0 <- tmp$pivot[sing.index]  # need the location of the pivots / equations
+
+##         w[jIx0] <- 1
+##         if(jIx0>1 && jIx0<(m+1))
+##           {
+##             jA <- U[1:(jIx0-1), 1:(jIx0-1)]
+##             jb <- U[1:(jIx0-1), sing.index]  # this is the singular part,
+##                                              # so it is at the end of the matrix
+
+##             jy <- qr.solve(-jA, jb, tol=1e-12)
+##             w[1:(jIx0-1)] <- jy
+##           }
+
+##         # now find the orthonormal basis for the w_i... w_qi
+##         w0 <- crossprod(UT[[i]], w)
+##         w1 <- w0/sqrt(sum(w0^2))
+
+##      # This is a bit of a "cheat", but it lets us trap occurences
+##      # where the solution is degenerate or the QR solution is fragile.  The
+##      # w converge quickly (within the first 100 iterations, so reusing the
+##      # previous ones in this error trap is probably OK.  If nothing
+##      # else, it increases the numerical stability of the algorithm
+
+##         # Check that we were able to get a new orthnormal basis.  If
+##         # not, reuse the last one and find its QR
+
+##         if(is.na(sum(w1))==TRUE)
+##         {    W <- Wout[[i]]
+##           } else {
+##               W <- qr.Q(qr(w1, tol=1e-12), complete=T)
+##           }
+
+##         # Now trap that we got a valid QR.
+
+##         if(is.numeric(W)==FALSE) W <- Wout[[i]]
+
+##         gkb <- matrix(0, n0[i], 1)
+##         jstd <- sqrt(1/df)
+##         if(n0[i]>1)
+##           {
+##             gkb[2:n0[i],1] <- jstd*rnorm(n0[i]-1)
+##           }
+##         jr <- jstd*rnorm(df+1)
+##         gkb[1,1] <- ifelse(runif(1)<0.5, sqrt(crossprod(jr)),
+##                            -1*sqrt(crossprod(jr)))
+##         A0gbs[,i] <- UT[[i]]%*%(W%*%gkb)
+##         Wout[[i]] <- W
+##       }
+##     return(list(A0gbs=A0gbs, W=Wout))
+
+##   }
 
